#!/usr/bin/perl 

use Net::SNMP;

#################################################################################
#                                                                               #
#       apcsnmp.sh                                                              #
#                                                                               #
#                                                                               #
#       version 2.31 - 1 March 2005                                             #
#                                                                               #
#       Copyright (C) 2005 Mike Dingeldey (portions of script)                  #
#                                       mdingeldey@sbcglobal.net                #
#                                                                               #
#       LICENSE                                                                 #
#       This program is free software; you can redistribute it and/or modify    #
#       it under the terms of the GNU General Public License as published by    #
#       the Free Software Foundation; either version 2 of the License, or       #
#       (at your option) any later version.                                     #
#                                                                               #
#       This program is distributed in the hope that it will be useful,         #
#       but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#       GNU General Public License for more details.                            #
#                                                                               #
#       You should have received a copy of the GNU General Public License       #
#       along with this program; if not, write to the Free Software             #
#       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA                #
#       02111-1307  USA                                                         #
#                                                                               #
#       If you don't know of the GPL, please refer to the file COPYING that     #
#       should have accompanied this script. If the file is unavailble, please  #
#       refer to:                                                               #
#                                                                               #
#       http://www.gnu.org/licenses/gpl.txt                                     #
#                                                                               #
#                                                                               #
#       The only thing that I ask is that if this script is modified (and you   #
#       think it may be useful to someone else) that you post it to deadcat.    #
#                                                                               #
#       This script was run using Fedora Core 3 and Net-SNMP 5.2.1. SNMP that   #
#       was installed with FC3 was removed and replaced by Net-SNMP.            #
#       APC MIBs were not installed. Instead, the OIDs were used.               #
#                                                                               #
#										#	
#	**** NOTE ****								#
#										#
#	Since the "up-front" part of the script was getting a bit too big, it	#
#	was cut off and placed in the README file. Please refer to the readme	#
#	file for what is monitored and which keywords need to be included in	#
#	your bb-hosts file.							#
#										#
#################################################################################


#################################################################################
#                                                                               #
# Our definitions                                                               #
#                                                                               #
#       $BBHOME      - The path to BB.						#
#       $BBDISP      - Our BB Display                                           #
#       $COmmunity   - Our SNMP read string (hopefully it isn't PUBLIC)         #
#       $VERBOSE     - A troubleshooting flag (verbose mode).			#
#                      Verbose mode will fill BBOUT and BBOUT.OLD.		#
#                      files. The default is 0 (FALSE). Use 1 for TRUE.         #
#       $LowVoltage  - The value used for low utility voltage (a sag).          #
#       $HighVoltage - The value used for an overvoltage condition.             #
#       $ChargeLevel - The level for a "low" battery charge (red alarm).	#
#       $MODCHARGE   - The value used for a moderate battery charge level.      #
#       $BADTEMP     - The bad UPS temperature limit. (Hotter = red alarm).	#
#	$GoodLoad    - A load <=GoodLoad and >=BadLoad is a yellow alarm.	#
#	$BadLoad     - A UPS load greater than this (%) is a red alarm.		#
#	$ThisRev     - A poor man's RCS. This is the firmware rev taken from	#
#		       one of your operational UPSes so that you can make sure  #
#		       that the same firmware is being used.....		#
#                                                                               #
#################################################################################
$BBHOME         = "/usr/local/xymon/server";
$BBDISP         = "149.155.16.56";
$Community	= "public";
$Verbose	= 0;
$LowVoltage     = 230;
$HighVoltage    = 255;
$ChargeLevel	= 50;
$BadTemp	= 40;
$GoodLoad	= 85;
$BadLoad	= 90;
$ThisAOS	= "v3.7.3";     # e.g. "v2.6.4"
$ThisAPP	= "v3.7.2";     # e.g. "v2.6.1"


#################################################################################
#										#
# OID Definitions								#
#										#
# I'll let the names speakfor themselves.					#
#										#
# If you need information about MIBs or OIDs that you want to use, refer to	#
# the website   www.mibdepot.com.   It is an excellent site and probably has	#
# the scoop on that MIB/OID you have been wanting to use....			#
#										#
#################################################################################
$upsBasicIdentModel             = ".1.3.6.1.4.1.318.1.1.1.1.1.1.0";
$upsBasicIdentName              = ".1.3.6.1.4.1.318.1.1.1.1.1.2.0";
$upsAdvIdentSerialNumber        = ".1.3.6.1.4.1.318.1.1.1.1.2.3.0";
$upsAdvIdentFirwareRevision     = ".1.3.6.1.4.1.318.1.1.1.1.2.1.0";
$upsBasicBatteryStatus          = ".1.3.6.1.4.1.318.1.1.1.2.1.1.0";
$upsBasicBatteryLastReplaceDate = ".1.3.6.1.4.1.318.1.1.1.2.1.3.0";
$upsAdvBatteryCapacity          = ".1.3.6.1.4.1.318.1.1.1.2.2.1.0";
$upsAdvBatteryTemperature       = ".1.3.6.1.4.1.318.1.1.1.2.2.2.0";
$upsAdvBatteryReplaceInidcator  = ".1.3.6.1.4.1.318.1.1.1.2.2.4.0";
$upsAdvBatteryNumOfBattPacks    = ".1.3.6.1.4.1.318.1.1.1.2.2.5.0";
$upsAdvBatteryNumOfBadBattPacks = ".1.3.6.1.4.1.318.1.1.1.2.2.6.0";
$upsAdvLineInputVoltage         = ".1.3.6.1.4.1.318.1.1.1.3.2.1.0";
$upsBasicOutputStatus           = ".1.3.6.1.4.1.318.1.1.1.4.1.1.0";
$upsAdvOutputLoad               = ".1.3.6.1.4.1.318.1.1.1.4.2.3.0";
$upsAdvTestLastDiagnosticsDate  = ".1.3.6.1.4.1.318.1.1.1.7.2.4.0";
$upsAdvTestDiagnosticsResults   = ".1.3.6.1.4.1.318.1.1.1.7.2.3.0";
$upsAdvTestCalibrationResults   = ".1.3.6.1.4.1.318.1.1.1.7.2.6.0";
$upsAdvTestCalibrationDate      = ".1.3.6.1.4.1.318.1.1.1.7.2.7.0";
$upsEnvironRelativeHumidity     = ".1.3.6.1.4.1.318.1.1.2.1.2.0";
$upsEnvironAmbientTemperature   = ".1.3.6.1.4.1.318.1.1.2.1.1.0";
$AOSrev				= ".1.3.6.1.4.1.318.1.4.2.4.1.4.1";
$APPrev				= ".1.3.6.1.4.1.318.1.4.2.4.1.4.2";


#################################################################################
#										#
# Open the bb-hosts file.							#
#										#
#################################################################################
open (FILE, "$BBHOME/etc/hosts.cfg") || die "\n\n Could not open the file '$BBHOME/etc/bb-hosts'\n\n";


#################################################################################
#										#
# Skip blank and comment lines, group, page and group-only lines.		#
#										#
#################################################################################
$SNMPerror = 0;

INPUT:	while (<FILE>) {

	#
	# Skip any blank and comment lines
	# Skip any group and group-only lines
	# Skip any page lines
	# Skip any subparent lines
	# Remove the newline character
	#
	next INPUT if /page/; 
	next INPUT if /^$/;
	next INPUT if /^#/;
	next INPUT if /group-only/;
	next INPUT if /subparent/;
	chomp;

	#################################################################################
        #										#
	# Check if the keyword of VOLT exists						#
	#										#
	# If found, get the voltage info and include it in a report.			#
        #										#
        #################################################################################
	if ( $_ =~ /[ \t]volt\b/ ) {

	  ($ipaddress, $hostname) = split(' ', $_);

	  if ( $SNMPerror == 0 ) {

	    ( $Info, $COLOR ) = GetVoltage($ipaddress);

            if ( $Info !~ /ERROR/ ) {
              HeaderOutput("$hostname.volt", $COLOR, $Info, $Verbose);
	    }

	    else {
	      #
	      # If there are problems communicating with the current host, change the color
	      # to blue and print an error message that will appear in BBOUT and in the status
	      # displays for this device.
	      #
	      # To speed up execution (especially if we have a bad card) set a flag that we will
	      # check in all other tests.
	      #
	      $SNMPerror = 1;
              $COLOR = "blue";
              $ErrorLine = "\n\n&blue There is a problem communicating via SNMP with $hostname at $ipaddress.\n";
              $ErrorLine = "$ErrorLine"."   Please check the SNMP management card and the settings for this device.\n\n\n";
              HeaderOutput("$hostname.volt", $COLOR, $ErrorLine, $Verbose);
            }
	  }
	  else {
            #
	    # Set the color to blue and print the error message
	    #
            $COLOR = "blue";
            HeaderOutput("$hostname.volt", $COLOR, $ErrorLine, $Verbose);
	  }
	}

        #################################################################################
        #										#
        # Check if the keyword of load exists.						#
        # If found, do an snmpget to determine the load percentage of the UPS.		#
        #										#
        #################################################################################
        if ( $_ =~ /[ \t]load\b/ ) {
          ($ipaddress, $hostname) = split(' ', $_);

	  if ( $SNMPerror == 0 ) {
            $Value = GetValue( $ipaddress, $Community, $upsAdvOutputLoad);

            if ( $Value !~ /ERROR/ ) {

	      #
	      # If load percentage shows as zero, modify it....
	      #
	      if ( $Value == 0 ) {
		$Value++; 
	      }

	      if ( $Value <= $GoodLoad ) {
		$COLOR = "green";
		$Info = "\n&green UPS is operating at $Value % capacity (GOOD)\n\n";
	      }
	      elsif (( $Value > $GoodLoad) && ( $Value <= $BadLoad )) {
		$COLOR = "yellow";
		$Info = "\n&yellow UPS is operating at $Value % capacity (CAUTION)\n\n";
	      }
	      else {
		$COLOR = "red";
		$Info = "\n&red UPS is operating at $Value % capacity (WARNING)\n\n";
	      }

              #
              # Send out output to BB.
              #
              HeaderOutput("$hostname.load", $COLOR, $Info, $Verbose);
	    }             

            else {
              #
              # If there are problems communicating with the current host, change the color
              # to blue and print an error message.
	      #
              # To speed up execution (especially if we have a bad card) set a flag that we will
	      # check in all other tests.
	      #
              $SNMPerror = 1;
              $COLOR = "blue";
              $ErrorLine = "\n\n&blue There is a problem communicating via SNMP with $hostname at $ipaddress.\n";
              $ErrorLine = "$ErrorLine"."   Please check the SNMP management card and the settings for this device.\n\n\n";
	      HeaderOutput("$hostname.load", $COLOR, $ErrorLine, $Verbose);
            }
	  }
	  else {      
            #
            # Set the color to blue and print the error message
            #
            $COLOR = "blue";
            HeaderOutput("$hostname.load", $COLOR, $ErrorLine, $Verbose);
          }
	}

	#################################################################################
	#										#
 	# Check if the keyword of charge exists.					#
 	# If found, do an snmpget to determine the charge of the battery.		#
 	#										#
 	#################################################################################
        if ( $_ =~ /[ \t]charge\b/ ) {
          ($ipaddress, $hostname) = split(' ', $_);

          if ( $SNMPerror == 0 ) {
            $Value = GetValue( $ipaddress, $Community, $upsAdvBatteryCapacity);

            if ( $Value !~ /ERROR/ ) {
              if ( $Value == 100 ) {
	        $Info = "\n&green Battery charge is $Value% (EXCELLENT)\n\n";
              }
              elsif ( $Value >= $ChargeLevel ) {
                $COLOR = "yellow";
                $Info = "\n&yellow Battery charge is $Value% (GOOD)\n\n";
              }
              else {
                $COLOR = "red";
                $Info = "\n&red Battery charge is $Value% (LOW)\n\n";
              }

              #
              # Send out output to BB.
              #
              HeaderOutput("$hostname.charge", $COLOR, $Info, $Verbose);
            }

            else {
              #
              # If there are problems communicating with the current host, change the color
              # to blue and print an error message.
              #
              # To speed up execution (especially if we have a bad card) set a flag that we will
              # check in all other tests.
              #
              $SNMPerror = 1;
              $COLOR = "blue";
              $ErrorLine = "\n\n&blue There is a problem communicating via SNMP with $hostname at $ipaddress.\n";
              $ErrorLine = "$ErrorLine"."   Please check the SNMP management card and the settings for this device.\n\n\n";
              HeaderOutput("$hostname.charge", $COLOR, $ErrorLine, $Verbose);
            }
          }

          else {
            #
            # Set the color to blue and print the error message
            #
            $COLOR = "blue";
            HeaderOutput("$hostname.charge", $COLOR, $ErrorLine, $Verbose);
          }
        }


        #################################################################################
        # 										#
        # Check if the keyword temp exists.						#
        #										#
        # If found, do an snmpget to determine the charge of the battery.		#
	#										#
	# I am using the battery temperature as a poor-man's indicator because the	#
	# 1400/2200/3000 don't report temperature - unless an environmental card	#
	# is installed in the UPS.							#
	#										#
	# I am also taking the liberty of converting the temperature. If you don't use	#
	# F, make the appropriate modifications....					#
	#										# 
	# The checks for the "proper" temperature OID will be included in a later 	#
	# rev (hopefully).								#
        #										#
        #################################################################################
        if ( $_ =~ /[ \t]temp\b/ ) {
          ($ipaddress, $hostname) = split(' ', $_);

          if ( $SNMPerror == 0 ) {
            $Value = GetValue( $ipaddress, $Community, $upsAdvBatteryTemperature);

            if ( $Value !~ /ERROR/ ) {

              ##$Value = ( $Value * 1.8 ) + 32 ;

              #########################################
              # **NOTE **   Temperature alarms will   #
              # be treated as a yellow alarm - this   #
              # is because some of us have UPSes in   #
              # areas that are hot. Also, the UPS     #
              # will continue to function - it just   #
              # won't provide any battery backup.     #
              #                                       #
              # If this is a problem for you, add     #
              # some more code to change the          #
              # operational parameters/alarms. If     #
              # you do add code, please don't post    #
              # this as a "change" or "update".       #
              #                                       #
              #########################################
	      if ( $Value <= $BadTemp ){
                $COLOR = "green";
                $Info = "\n&green UPS Temperature is $Value C (OK)\n\n";
	      }
	      else {
                $COLOR = "yellow";
                $Info = "\n&yellow UPS Temperature is $Value C (HOT)\n\n";
	      }

              #
              # Send out output to BB.
              #
              HeaderOutput("$hostname.temp", $COLOR, $Info, $Verbose);
            }

            else {
              #
              # If there are problems communicating with the current host, change the color
              # to blue and print an error message.
              #
              # To speed up execution (especially if we have a bad card) set a flag that we will
              # check in all other tests.
              #
              $SNMPerror = 1;
              $COLOR = "blue";
              $ErrorLine = "\n\n&blue There is a problem communicating via SNMP with $hostname at $ipaddress.\n";
              $ErrorLine = "$ErrorLine"."   Please check the SNMP management card and the settings for this device.\n\n\n";
              HeaderOutput("$hostname.temp", $COLOR, $ErrorLine, $Verbose);
            }
          }

          else {
            #
            # Set the color to blue and print the error message
            #
            $COLOR = "blue";
            HeaderOutput("$hostname.temp", $COLOR, $ErrorLine, $Verbose);
          }
        }

        #################################################################################
        #										#
        # Check if the keyword of replace exists.					#
	#										#
        # If found, do an snmpget to determine the battery replacement status and	#
	# the date of last battery replacement. Also check if there are any external	#
	# battery packs and the replacement status of those.				#
        #										#
	# Because of the number of gets required by this section, place it last in	#
	# the script.									#
        #################################################################################
        if ( $_ =~ /[ \t]replace\b/ ) {
          ($ipaddress, $hostname) = split(' ', $_);

          if ( $SNMPerror == 0 ) {
            $Value = GetValue( $ipaddress, $Community, $upsAdvBatteryReplaceInidcator);

            if ( $Value !~ /ERROR/ ) {

	      #
	      # Get the battery replacement date.
	      # Get the number external battery packs attached.
	      #
              $ExtPacks = GetValue( $ipaddress, $Community, $upsAdvBatteryNumOfBattPacks);
              $ReplaceDate = GetValue( $ipaddress, $Community, $upsBasicBatteryLastReplaceDate);

              if ( $Value == 1) {
                $COLOR = "green";
	        $Info = "\n&green Internal batteries are OK\n";
              }
              else {
                $COLOR = "red";
	        $Info = "\n&red Internal batteries are bad (REPLACE!)\n";
	      }

              $Info = "$Info"."\nBatteries were last replaced on: $ReplaceDate\n";

	      if ( $ExtPacks > 0 ) {
	        #
	        # Check if there are any bad external packs.
	        #
	        $BadPacks = GetValue( $ipaddress, $Community, $upsAdvBatteryNumOfBadBattPacks);

	        if ( $BadPacks > 0 ) {
	          $COLOR = "red";
                  $Info = "$Info"."\n&red Replacement required for: $BadPacks external battery packs.\n";
		}
		else {
                  $Info = "$Info"."\n&green External battery packs are OK.\n";
		}
              }

              #
              # Send out output to BB.
              #
              HeaderOutput("$hostname.replace", $COLOR, $Info, $Verbose);
            }

            else {
              #
              # If there are problems communicating with the current host, change the color
              # to blue and print an error message.
              #
              # To speed up execution (especially if we have a bad card) set a flag that we will
              # check in all other tests.
              #
              $SNMPerror = 1;
              $COLOR = "blue";
              $ErrorLine = "\n\n&blue There is a problem communicating via SNMP with $hostname at $ipaddress.\n";
              $ErrorLine = "$ErrorLine"."   Please check the SNMP management card and the settings for this device.\n\n\n";
              HeaderOutput("$hostname.replace", $COLOR, $ErrorLine, $Verbose);
            }
          }

          else {
            #
            # Set the color to blue and print the error message
            #
            $COLOR = "blue";
            HeaderOutput("$hostname.replace", $COLOR, $ErrorLine, $Verbose);
          }
        }

        #################################################################################
        #                                                                               #
        # Check if the keyword of test exists.                                          #
        # If found, do an snmpget to determine the status of the last self-test.	#
        #                                                                               #
        #################################################################################
        if ( $_ =~ /[ \t]test\b/ ) {
          ($ipaddress, $hostname) = split(' ', $_);

          if ( $SNMPerror == 0 ) {

	    ( $Info, $COLOR ) = GetTestInfo( $ipaddress );

            if ( $Info !~ /ERROR/ ) {
              HeaderOutput("$hostname.test", $COLOR, $Info, $Verbose);
	    }
            else {
              #
              # If there are problems communicating with the current host, change the color
              # to blue and print an error message.
              #
              # To speed up execution (especially if we have a bad card) set a flag that we will
              # check in all other tests.
              #
              $SNMPerror = 1;
              $COLOR = "blue";
              $ErrorLine = "\n\n&blue There is a problem communicating via SNMP with $hostname at $ipaddress.\n";
              $ErrorLine = "$ErrorLine"."   Please check the SNMP management card and the settings for this device.\n\n\n";
              HeaderOutput("$hostname.test", $COLOR, $ErrorLine, $Verbose);
            }
          }

          else {
            #
            # Set the color to blue and print the error message
            #
            $COLOR = "blue";
            HeaderOutput("$hostname.test", $COLOR, $ErrorLine, $Verbose);
          }
        }


        #################################################################################
        #                                                                               #
        # Check if the keyword of rev exists.						#
        # If found, do an snmpget to determine the firmware revision.			#
        #                                                                               #
	# This has been fixed to pull the running versions via snmp from the card.	#
	# Make sure that the current AOS/APP revs are set in the ThisAOS and ThisAPP	#
	# variables at the beginning of the script.					#
	#										#
        #################################################################################
        if ( $_ =~ /[ \t]rev\b/ ) {
          ($ipaddress, $hostname) = split(' ', $_);

          if ( $SNMPerror == 0 ) {

	    ( $Info, $COLOR ) = GetRev( $ipaddress );

            if ( $Info !~ /ERROR/ ) {
              HeaderOutput("$hostname.rev", $COLOR, $Info, $Verbose);
            }

            else {
              #
              # If there are problems communicating with the current host, change the color
              # to blue and print an error message.
	      #
              # To speed up execution (especially if we have a bad card) set a flag that we will
              # check in all other tests.
              #
              $SNMPerror = 1;
              $COLOR = "blue";
              $ErrorLine = "\n\n&blue There is a problem communicating via SNMP with $hostname at $ipaddress.\n";
              $ErrorLine = "$ErrorLine"."   Please check the SNMP management card and the settings for this device.\n\n\n";
              HeaderOutput("$hostname.rev", $COLOR, $ErrorLine, $Verbose);
            }
          }

	  else {
            #
            # Set the color to blue and print the error message
            #
            $COLOR = "blue";
            HeaderOutput("$hostname.rev", $COLOR, $ErrorLine, $Verbose);
          }
        }


        #################################################################################
        #                                                                               #
        # Check if the keyword of stats exists.                                         #
        #                                                                               #
        # If found, do an snmpget to retrieve the operational status of the UPS,        #
        # as well a providing information on some key operational values (granted,      #
        # some are shown elsewhere, but I think it is a good idea to show them here     #
        # too).                                                                         #
        #                                                                               #
        # Information included in this "view" include:                                  #
	# UPS model number								#
        # UPS Serial Number                                                             #
        # AOS/APP software rev levels                                                   #
        # Last Calibration Date and Calibration Results                                 #
        # Last Diagnostic Date and Diagnostics Results                                  #
        # Utility (incoming) line voltage.						#
        #                                                                               #
        #################################################################################
        if ( $_ =~ /[ \t]stats\b/ ) {
          ($ipaddress, $hostname) = split(' ', $_);

          if ( $SNMPerror == 0 ) {
            $Value = GetValue( $ipaddress, $Community, $upsBasicOutputStatus);

            if ( $Value !~ /ERROR/ ) {
              if ( $Value == 2 ) {
                $COLOR = "green";

                $ModelName    = GetValue( $ipaddress, $Community, $upsBasicIdentModel);
                $SerialNumber = GetValue( $ipaddress, $Community, $upsAdvIdentSerialNumber);
 
		$Info = "\nModel Number:  $ModelName\nSerial Number: $SerialNumber\n";
                $Info = "$Info"."\n&green $hostname is operating correctly.\n";


		( $RevInfo, $newCOLOR ) = GetRev ( $ipaddress );     
		if ( $newCOLOR ne "green" ) { $COLOR = $newCOLOR; }
		( $TestInfo, $newCOLOR ) = GetTestInfo ( $ipaddress );
                if ( $newCOLOR ne "green" ) { $COLOR = $newCOLOR; }
		( $VoltInfo, $newCOLOR) = GetVoltage ( $ipaddress );
                if ( $newCOLOR ne "green" ) { $COLOR = $newCOLOR; }

		$Info = "$Info"."$RevInfo"."$TestInfo"."$VoltInfo";
             }

              #
              # on SmartBoost (undervoltage)
              #
              elsif ( $Value == 4 ) {
                $COLOR = "yellow";
                $Info = "\n\n&yellow SmartBoost is operating (LOW Utility Voltage)\n\n";
              }

              #
              # on SmartTrim (overvoltage)
              #
              elsif ( $Value == 12 ) {
                $COLOR = "yellow";
                $Info = "\n\n&yellow SmartTrim is operating (HIGH Utility Voltage)\n\n";
              }

              #
              # The unit is running on battery (quick! plug it in! :) )
              #
              elsif ( $Value == 3 ) {
                $COLOR = "red";
                $Info = "\n\n&red The UPS is running on battery!\n\n";
              }

              #
              # Either the UPS is rebooting, or the snmp card is rebooting.
              #
              elsif ( $Value == 8 ) {
                $COLOR = "red";
                $Info = "\n\n&red The management card or UPS is currently rebooting.\n\n";
              }

              #
              # The unit has run for so long on battery that the batteries are exhausted and the
              # ups has shutdown (it will wake up when power is restored).
              #
              elsif ( $Value == 11 ) {
                $COLOR = "red";
                $Info = "\n\n&red The UPS has reached its backup life and has been shutoff until power returns.\n\n";
              }

              #
              # The unit is in software bypass mode
              #
              elsif ( $Value == 6 ) {
                $COLOR = "red";
                $Info = "\n\n&red The UPS is in software bypass mode.\n\n";
              }

              #
              # The unit is in switched bypass mode
              #
              elsif ( $Value == 9 ) {
                $COLOR = "red";
                $Info = "\n\n&red The UPS is in switched bypass mode.\n\n";
              }

              #
              # The unit is in bypass mode because of a hardware failure
              #
              elsif ( $Value == 10 ) {
                $COLOR = "red";
                $Info = "\n\n&red The UPS is in hardware bypass mode.\n\n";
              }

              #
              # The unit is in  timed sleepd mode (taking a nap)
              #
              elsif ( $Value == 5 ) {
                $COLOR = "red";
                $Info = "\n\n&red The UPS is in timed sleep mode.\n\n";
              }

              #
              # The UPS has been powered off (but the snmp card lives)
              #
              elsif ( $Value == 7 ) {
                $COLOR = "red";
                $Info = "\n\n&red The UPS is turned off.\n\n";
              }

              #
              # There is an unknown error.
              #
              else {
                $COLOR = "red";
                $Info = "\n\n&red The UPS is experiencing an unknown error!\n\n";
              }

              #
              # Send out output to BB.
              #
              HeaderOutput("$hostname.stats", $COLOR, $Info, $Verbose);
            }

            #
            # If there are problems communicating with the current host, change the color
            # to blue and print an error message.
            #
            # To speed up execution (especially if we have a bad card) set a flag that we will
            # check in all other tests.
            #
            else {
              $SNMPerror = 1;
              $COLOR = "blue";
              $ErrorLine = "\n\n&blue There is a problem communicating via SNMP with $hostname at $ipaddress.\n";
              $ErrorLine = "$ErrorLine"."   Please check the SNMP management card and the settings for this device.\n\n\n";
              HeaderOutput("$hostname.stats", $COLOR, $ErrorLine, $Verbose);
            }
          }
          else {
            #
            # Set the color to blue and print the error message
            #
            $COLOR = "blue";
            HeaderOutput("$hostname.stats", $COLOR, $ErrorLine, $Verbose);
          }
        }
} continue { $SNMPerror = 0; }

close (FILE);
exit 0;




#################################################################
#								#
# GetVoltage							#
#								#
# Perform a blocking SNMPv1 get-request to retrieve the value	#
# for the utility line voltage. Return a value of ERROR if an	#
# SNMP communications error occurs, or return a string 		#
# containing the voltage.					#
#								#
# Returned Values:						#
#	$info  - a string indicating the current input voltage  #
#		 and the appropriate html tag (the colorized	#
#		 "dot").					#
#	$color - the resulting color from the operation.	#
#		 two colors can be returned:			#
#			green - utility voltage is OK.		#
#			red   - bad voltage (over/under)	#
#								#
#################################################################
sub GetVoltage {

        my ($ipaddress) = @_;

	$voltage = GetValue( $ipaddress, $Community, $upsAdvLineInputVoltage);
            
	if ( $voltage !~ /ERROR/ ) {
	  if (( $voltage >= $LowVoltage ) || ( $voltage <= $HighVoltage )) {
	    $color = "green";
	    return( "\n&green Utility Voltage is $voltage Volts (OK)\n", $color );
	  }
	  else {
	    if ( $voltage > $HighVoltage ) {                                          
	      $color = "red"; 
	      return( "\n&red Utility Voltage is $voltage Volts (HIGH)\n", $color );          
            }
	    else {
	      $color = "red";
	      return( "\n&red Utility Voltage is $voltage Volts (LOW)\n", $color );
	    }
	  }
	}
	else {
	  return ( "ERROR", "blue" );
	}
}




#################################################################
#                                                               #
# GetRev                                                        #
#                                                               #
# Perform a blocking SNMPv1 get-request to retrieve the value   #
# for the AOS and APP rev levels.				#
#								#
# Returned Data:						#
#   $info  - a string indicating the AOS and APP rev levels.	#
#            included is the html tag (the "dot") of the	#
#	     appropriate color.					#
#   $color - the status color from out operation. The colors	#
#	     are as follows:					#
#		green  - good rev level (matches our rev)	#
#		yellow - does not match. an update in code is	#
#			 probably required.			#
#								#
#  If an SNMP error occurs during the SNMP get, a value of	#
#  "ERROR" is returned as the value of $info, and $color will	#
#  be returned as "blue".					#
#                                                               #
#################################################################
sub GetRev {

        my ($ipaddress) = @_;

        $AOS = GetValue( $ipaddress, $Community, $AOSrev);
              
        if ( $AOS !~ /ERROR/ ) {

          $APP = GetValue( $ipaddress, $Community, $APPrev);

          if ( $AOS eq $ThisAOS ) {
            $color = "green"; 
            $info = "\n&green AOS Software: $AOS\n";
          }   
          else {
            $color = "green";
            $info = "\n&green AOS Software: $AOS\n";
          }   
          if ( $APP eq $ThisAPP ) {
            $info = "$info"."&green APP Software: $APP\n";
          }   
          else {
            $color = "green";
            $info =  "$info"."&green APP Software: $APP\n";  
          } 

	  return ( $info, $color );

	}
	else {
	  #return ( "ERROR", "blue" );
	  return ( "UNKNOW Software rev", "green" );
	}
}




#################################################################
#								#
# GetTestInfo							#
#								#
# Perform a blocking SNMPv1 get-request to retrieve the value   #
# for the calibration and diagnostics test.			#
#                                                               #
# Returned Data:                                                #
#   $info  - a string indicating the state and success of the	#
#	     calibration and diagnostics tests.	included in 	#
#	     returned string is the html tag (the "dot") of 	#
#	     the appropriate color (see color for values).      #
#   $color - the status color from out operation. The colors    #
#            are as follows:                                    #
#               green  - successful test.		        #
#               yellow - bad / unsuccessful test.		#
#			 this value is yellow because i have	#
#			 seen the diagnostics test show as 	#
#			 "failed" after calibration od the ups	#
#			 is successful. the "failed" test	#
#			 clears when you rerun it. APC BUG?	#
#                                                               #
#  If an SNMP error occurs during the SNMP get, a value of      #
#  "ERROR" is returned as the value of $info, and $color will   #
#  be returned as "blue".                                       #
#                                                               #
#################################################################
sub GetTestInfo {

	my ( $ipaddress ) = @_;

        $DiagResults = GetValue( $ipaddress, $Community, $upsAdvTestDiagnosticsResults);

        if ( $DiagResults !~ /ERROR/ ) {
            
          $CalResults = GetValue( $ipaddress, $Community, $upsAdvTestCalibrationResults);
          $DiagDate = GetValue( $ipaddress, $Community, $upsAdvTestLastDiagnosticsDate);
          $CalDate = GetValue( $ipaddress, $Community, $upsAdvTestCalibrationDate);
              
          #
          # Get rid of any newlines that might be tagging along
          # 
          chomp($CalDate);
          chomp($DiagDate);
              
          if ( $DiagResults == 1 ) {
            if ( $DiagDate ne "Unknown" ) {
               $color = "yellow";
               $info = "\n&green Last Diagnostics Date:  $DiagDate    PASSED\n";
            } 
            else {
               $color = "yellow";
               $info = "\n&red Last Diagnostics Date:  $DiagDate    \t UNKNOWN\n";
            } 
          } 
          else {
            $color = "yellow";
                
            if ( $DiagResults == 2 ) {
              $info = "\n&yellow Last Diagnostics Date:  $DiagDate    FAILED\n";
            }
            elsif ( $DiagResults == 3) {
              $info = "\n&yellow Last Diagnostics Date:  $DiagDate    INVALID TEST\n";
            }
            else {
              $info = "\n&yellow Last Diagnostics Date:  $DiagDate    IN PROGRESS\n";
            }
          }

          #print "$hostname Cal result = $CalResults\n";
          #if ( $CalResults eq "ok" ) {
          if ( $CalResults == 1 ) {
            if ( $CalDate ne "Unknown" ) {
              if ( $DiagResults == 1 ) {
                $color = "green";
                $info = "$info"."&green Last Calibration Date:  $CalDate    PASSED\n";
              }
              else {
                $color = "yellow";
                $info = "$info"."&yellow Last Calibration Date:  $CalDate    \t UNKNOWN\n";
              }
            }
            else {
              $info = "$info"."&yellow Last Calibration Date:  $CalDate    \t UNKNOWN\n";
            }
          }
          else {
            if ( $CalResults eq "INVALID"  ) {
              $info = "$info"."&yellow Last Calibration Date:  $CalDate    INVALID\n";
            }
            else {
              $info = "$info"."&green Last Calibration Date:  $CalDate    IN PROGRESS\n";
            }
          }
	
	  return ( $info, $color );

	}
	else {
	  return ( "ERROR", "blue" );
	}
}



#################################################################
#                                                               #
# GetValue							#
#                                                               #
# Get the passed OID value from the specified IP address, and   #
# pass back the value.						#
#								#
# This is performed using a Blocking SNMPv1 get-request because #
# The APC cards do not seem to respond to SNPMv2 get requests.	#
# If they did, we could use a non-blocking get-request and 	#
# process more than single requests.				#
#								#
# If there is an error, return the value "ERROR" so that we	#
# know to avoid it.						#
#                                                               #
#################################################################
sub GetValue {

	my ($ipaddress, $cstring, $oid) = @_;

	#
	# Create our session.
	# 
	# If you have problems with bogus BLUE alarms showing up,
	# you may want to tweak the timeout/retry settings.
	# This is a tweak at own risk, because it will affect how
	# this script runs (and how fast it does it...)
	#
	# As it is, our defaults are set to a timeout of 2s
	# (timeouts are in whole seconds only) and three retries.
	#
	my ($session, $error) = Net::SNMP->session (
				-hostname  => $ipaddress,
				-community => $cstring,
				-port      => 161,
				-timeout   => 2,
				-retries   => 3 );

	# See if there are any errors opening our session. If so,
	# print an error message and exit. If the session can't be
	# opened, then there are bigger problems...
	#
	if (!defined($session)) {
	  printf("ERROR: %s.\n", $error);
	  exit 1;
        }

	#
	# If the seeion was created, send our "request" to the target
	# device. If there is an error condition, return the string
	# "ERROR" to the caller and have the caller print an error
	# message.  ** NOTE ** - the error message will appear in the
	# BBOUT file. If desired, we can send the error to BB and have it
	# appear on the display.
	#
	# Returning the error value will let us know to skip everything
	# else associated with the offending IP address. Thsi will allow
	# the rest of the data from the bb-hosts file to be processed.
	#
	my $result = $session->get_request( -varbindlist => [$oid]);

	$session->close;
	if (!defined($result)) {
	  return ( "ERROR" );
	}
	else {
          return ( $result->{$oid} );
	}
}



#################################################################
#                                                               #
# HeaderOutput                                                  #
#                                                               #
# Send out final information to BB.                             #
#                                                               #
#################################################################
sub HeaderOutput {

        my($host, $color, $info, $verbose) = @_;

        #########################################
        #                                       #
        # Get our timestamp.                    #
        #                                       #
        #########################################
        my $datestamp = `date`;

        #########################################
        #                                       #
        # Let's send the header to BB           #
        #                                       #
        #########################################
        `$BBHOME/bin/bb $BBDISP "status $host $color $datestamp $info"`;
        if ( $verbose) {
          print "status $host $color $datestamp $info";
        }
}
